Imports System.Runtime.InteropServices

Public Class WNET
    <DllImport("mpr.dll", CharSet:=CharSet.Auto)> _
    Private Shared Function WNetEnumResource(ByVal hEnum As IntPtr, ByRef lpcCount As Integer, ByVal lpBuffer As IntPtr, ByRef lpBufferSize As Integer) As Integer
    End Function

    <DllImport("mpr.dll", CharSet:=CharSet.Auto)> _
    Private Shared Function WNetOpenEnum(ByVal dwScope As RESOURCE_SCOPE, ByVal dwType As RESOURCE_TYPE, ByVal dwUsage As RESOURCE_USAGE, ByRef lpNetResource As NETRESOURCE, ByRef lphEnum As IntPtr) As Integer
    End Function

    <DllImport("mpr.dll", CharSet:=CharSet.Auto)> _
    Private Shared Function WNetGetResourceParent(ByRef lpNetResource As NETRESOURCE, ByVal lpBuffer As IntPtr, ByRef lpBufferSize As Integer) As Integer
    End Function

    <DllImport("mpr.dll", CharSet:=CharSet.Auto)> _
    Private Shared Function WNetCloseEnum(ByVal hEnum As IntPtr) As Integer
    End Function


    Private Enum RESOURCE_SCOPE
        CONNECTED = &H1
        GLOBALNET = &H2
        REMEMBERED = &H3
        RECENT = &H4
        CONTEXT = &H5
    End Enum
    Private Enum RESOURCE_TYPE
        ANY = &H0
        DISK = &H1
        PRINT = &H2
        RESERVED = &H8
    End Enum
    Private Enum RESOURCE_USAGE
        CONNECTABLE = &H1
        CONTAINER = &H2
        NOLOCALDEVICE = &H4
        SIBLING = &H8
        ATTACHED = &H10
        ALL = (CONNECTABLE Or CONTAINER Or ATTACHED)
    End Enum
    Private Enum RESOURCE_DISPLAYTYPE
        GENERIC = &H0
        DOMAIN = &H1
        SERVER = &H2
        SHARE = &H3
        FILE = &H4
        GROUP = &H5
        NETWORK = &H6
        ROOT = &H7
        SHAREADMIN = &H8
        DIRECTORY = &H9
        TREE = &HA
        NDSCONTAINER = &HB
    End Enum
    Private Enum NERR
        Success = 0
        MORE_DATA = 234
        NO_BROWSER_SERVERS_FOUND = 6118
        INVALID_LEVEL = 124
        ACCESS_DENIED = 5
        INVALID_PARAMETER = 87
        NOT_ENOUGH_MEMORY = 8
        NETWORK_BUSY = 54
        BAD_NETPATH = 53
        NO_NETWORK = 1222
        INVALID_HANDLE_STATE = 1609
        EXTENDED_ERROR = 1208
    End Enum

    Private Structure NETRESOURCE
        Public dwScope As RESOURCE_SCOPE
        Public dwType As RESOURCE_TYPE
        Public dwDisplayType As RESOURCE_DISPLAYTYPE
        Public dwUsage As RESOURCE_USAGE
        <MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPTStr)> Public lpLocalName As String
        <MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPTStr)> Public lpRemoteName As String
        <MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPTStr)> Public lpComment As String
        <MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPTStr)> Public lpProvider As String
    End Structure

    Private Shared FoundItems As ArrayList

    Private Shared Function WNETOE(ByVal o As Object) As Boolean
        Dim iRet As NERR
        Dim ptrHandle As IntPtr = New IntPtr()
        Dim I As Integer

        Try
            iRet = WNetOpenEnum(RESOURCE_SCOPE.GLOBALNET, RESOURCE_TYPE.DISK Or RESOURCE_TYPE.PRINT, RESOURCE_USAGE.ALL, o, ptrHandle)

            If (iRet <> NERR.Success) Then
                Exit Function
            End If

            Dim entries As Integer
            Dim buffer As Integer = 16384

            Dim ptrBuffer As IntPtr = Marshal.AllocHGlobal(buffer)

            Dim nr As NETRESOURCE

            Do
                entries = -1
                buffer = 16384

                iRet = WNetEnumResource(ptrHandle, entries, ptrBuffer, buffer)

                If ((iRet <> NERR.Success) Or (entries < 1)) Then
                    Exit Do
                End If

                Dim ptr As Integer = ptrBuffer.ToInt32()

                For I = 0 To entries - 1
                    nr = Marshal.PtrToStructure(New IntPtr(ptr), GetType(NETRESOURCE))
                    'MsgBox(Hex(nr.dwUsage))

                    ptr += Marshal.SizeOf(nr)
                    FoundItems.Add(nr.lpRemoteName)
                Next

            Loop

            Marshal.FreeHGlobal(ptrBuffer)

            iRet = WNetCloseEnum(ptrHandle)

        Catch ex As Exception
        End Try

        Return True
    End Function
    Private Shared Function WNETGP(ByVal o As Object) As String
        Dim iRet As NERR
        Dim I As Integer

        Dim buffer As Integer = 16384

        Dim ptrBuffer As IntPtr = Marshal.AllocHGlobal(buffer)

        Dim nr As NETRESOURCE

        buffer = 16384

        iRet = WNetGetResourceParent(o, ptrBuffer, buffer)

        If (iRet = NERR.Success) Then
            Dim ptr As Integer = ptrBuffer.ToInt32()
            nr = Marshal.PtrToStructure(New IntPtr(ptr), GetType(NETRESOURCE))
        End If

        Marshal.FreeHGlobal(ptrBuffer)

        Return nr.lpRemoteName
    End Function

    Public Shared Function GetProviders() As String()
        Dim i As Integer

        If FoundItems Is Nothing Then FoundItems = New ArrayList()
        FoundItems.Clear()
        If WNETOE(Nothing) = True Then
            For i = FoundItems.Count - 1 To 0 Step -1
                If FoundItems(i).ToString.StartsWith("Microsoft Windows") = False Then
                    FoundItems.RemoveAt(i)
                End If
            Next
            Return FoundItems.ToArray(GetType(String))
        Else
            Throw New Exception("Could not access resource")
        End If
    End Function

    Public Shared Function GetServers() As String()
        Dim deSearchPath As New DirectoryServices.DirectoryEntry("WinNT:")
        Dim deItem As DirectoryServices.DirectoryEntry

        If FoundItems Is Nothing Then FoundItems = New ArrayList()
        FoundItems.Clear()

        deSearchPath.Children.SchemaFilter.Add("Computer")
        deSearchPath.Children.SchemaFilter.Add("Domain")

        For Each deItem In deSearchPath.Children
            FoundItems.Add(deItem.Name)
        Next

        Return FoundItems.ToArray(GetType(String))

    End Function
    Public Shared Function GetSubItems(ByVal Server As String) As String()
        Dim nr As NETRESOURCE

        If FoundItems Is Nothing Then FoundItems = New ArrayList()
        FoundItems.Clear()

        nr.lpRemoteName = Server
        nr.dwScope = RESOURCE_SCOPE.GLOBALNET
        nr.dwUsage = RESOURCE_USAGE.CONTAINER

        If WNETOE(nr) = True Then
            Return FoundItems.ToArray(GetType(String))
        Else
            Throw New Exception("Could not access resource")
        End If
    End Function
    Public Shared Function GetParent(ByVal Item As String, ByVal Provider As String) As String
        Dim nr As NETRESOURCE

        If FoundItems Is Nothing Then FoundItems = New ArrayList()
        FoundItems.Clear()

        nr.lpRemoteName = Item
        nr.lpProvider = Provider

        Return WNETGP(nr)
    End Function
End Class
